import pickle
import os
import shutil

from logger.appLogger import AppLogger


class FileOperation:
    """
                This class shall be used to save the model after training
                and load the saved model for prediction.

                Written By: iNeuron Intelligence
                Version: 1.0
                Revisions: None

                """

    def __init__(self):
        self.file_object = open('logs/fileOperationLogs.txt', 'a+')
        self.logger_object = AppLogger()
        self.model_directory = 'models/'

    def save_model(self, model, filename):
        """
            Method Name: save_model
            Description: Save the model file to directory
            Outcome: File gets saved
            On Failure: Raise Exception

            Written By: iNeuron Intelligence
            Version: 1.0
            Revisions: None
"""
        self.logger_object.log(self.file_object, 'Entered the save_model method of the File_Operation class')
        try:
            path = os.path.join(self.model_directory, filename)  # create seperate directory for each cluster
            if os.path.isdir(path):  # remove previously existing models for each clusters
                shutil.rmtree(self.model_directory)
                os.makedirs(path)
            else:
                os.makedirs(path)  #
            with open(path + '/' + filename + '.sav',
                      'wb') as f:
                pickle.dump(model, f)  # save the model to file
            self.logger_object.log(self.file_object,
                                   'Model File ' + filename + ' saved. Exited the save_model method of the Model_Finder class')

            return 'success'
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in save_model method of the Model_Finder class. Exception message:  ' + str(
                                       e))
            self.logger_object.log(self.file_object,
                                   'Model File ' + filename + ' could not be saved. Exited the save_model method of the Model_Finder class')
            raise Exception()

    def load_model(self, filename):
        """
                    Method Name: load_model
                    Description: load the model file to memory
                    Output: The Model file loaded in memory
                    On Failure: Raise Exception

                    Written By: iNeuron Intelligence
                    Version: 1.0
                    Revisions: None
        """
        self.logger_object.log(self.file_object, 'Entered the load_model method of the File_Operation class')
        try:
            with open(self.model_directory + filename + '/' + filename + '.sav',
                      'rb') as f:
                self.logger_object.log(self.file_object,
                                       'Model File ' + filename + ' loaded. Exited the load_model method of the Model_Finder class')
                return pickle.load(f)
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in load_model method of the Model_Finder class. Exception message:  ' + str(
                                       e))
            self.logger_object.log(self.file_object,
                                   'Model File ' + filename + ' could not be saved. Exited the load_model method of the Model_Finder class')
            raise Exception()

    def find_correct_model_file(self, cluster_number):
        """
                            Method Name: find_correct_model_file
                            Description: Select the correct model based on cluster number
                            Output: The Model file
                            On Failure: Raise Exception

                            Written By: iNeuron Intelligence
                            Version: 1.0
                            Revisions: None
                """
        self.logger_object.log(self.file_object,
                               'Entered the find_correct_model_file method of the File_Operation class')
        try:
            self.cluster_number = cluster_number
            self.folder_name = self.model_directory
            self.list_of_model_files = []
            self.list_of_files = os.listdir(self.folder_name)
            for self.file in self.list_of_files:
                try:
                    if (self.file.index(str(self.cluster_number)) != -1):
                        self.model_name = self.file
                except:
                    continue
            self.model_name = self.model_name.split('.')[0]
            self.logger_object.log(self.file_object,
                                   'Exited the find_correct_model_file method of the Model_Finder class.')
            return self.model_name
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in find_correct_model_file method of the Model_Finder class. Exception message:  ' + str(
                                       e))
            self.logger_object.log(self.file_object,
                                   'Exited the find_correct_model_file method of the Model_Finder class with Failure')
            raise Exception()



import scipy
from scipy.stats.stats import pearsonr
from datetime import datetime
import numpy as np
import pandas as pd
import plotly
import plotly.graph_objs as go
import json
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns

from logger.appLogger import AppLogger


class DataVisualization:
    """
    This class shall be used to include all Data Visualization techniques to be feed to the Machine Learning Models

    Written By: Shakti Kumar
    Version: 1.0
    Revisions: None

    """

    def __init__(self):
        self.file_object = open('logs/dataVisualizationLogs.txt', 'a+')
        self.logger_object = AppLogger()

    def balance_imbalance_check(self, dataframe, target):

        """
        Method Name: balance_imbalance_check
        Description: This method will be used to plot the balance/imbalance datasets using barplot/countplot

        Input Description: data: the input dataframe with target column.
                           target: target column name.

        Output: plot of target variable value count.
        On Failure: Raise Exception

        Written By: shakti kumar
        Version: 1.0
        Revisions: None

        """

        try:
            # x = sns.countplot(target,data=dataframe).set_title("Balance Imbalance Count")
            # sns.barplot(x = 'is_promoted', y = 'is_promoted' ,data=df, hue  = 'is_promoted', estimator = lambda x: len(x)/len(df) *100).set_title("Balance Imbalance Count")
            sns.barplot(x='is_promoted', y='is_promoted', data=dataframe,
                        estimator=lambda x: len(x) / len(dataframe) * 100)
            plt.xlabel('ispromoted')
            plt.ylabel('percentage')
            plt.title('Balance Imbalance Count')
            plt.savefig("static/graphs/imbalance.png")

        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in checking balance and imbalance of the target(Graphically). Exception message: ' + str(
                                       e))  # Logging the exception message
            self.logger_object.log(self.file_object,
                                   'Target Value balance and imbalance check fail.Exited the balance_imbalance_check method of the DataVisualization class')  # Logging unsuccessful load of data
            raise Exception()  # raising exception and exiting

    def correlation_heatmap(self, dataframe):

        """
        Method Name: correlation_heatmap
        Description: This method will be used to generate interactive heatmap plot to show the pairwise correlation of input variables

        Input Description: data: the input dataframe with target column.
        Output: returns json file with correlation information that can be used by plotly to generate interactive plots.
        On Failure: Raise Exception

        Written By: Shashank Prakash
        Version: 1.0
        Revisions: None

        """
        try:

            data = dataframe.select_dtypes(include=[np.number])

            #plt.figure(figsize=(20, 10))
            #sns.set_palette("PuBuGn_d")
            #plot = sns.heatmap(data.corr(), annot=True, cmap='RdYlGn')
            #plot.figure.savefig("static/graphs/correlation.png")

            plotdata = [go.Heatmap(
                z=data.corr(),
                x=list(data.columns),
                y=list(data.columns)
                ,dx=1,dy=1
                )]
            graphJSON = json.dumps(plotdata, cls=plotly.utils.PlotlyJSONEncoder)

            return graphJSON




        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in plotting correlation heatmap. Exception message: ' + str(
                                       e))  # Logging the exception message
            self.logger_object.log(self.file_object,
                                   'Plotting of heatmap fails.Exited the correlation_heatmap method of the DataVisualization class')  # Logging unsuccessful load of data
            raise Exception()  # raising exception and exiting
